 --==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for CONV-BURST_CTRL design
--==============================================================================
--
-- author: Denia Bouhired (denia.bouhired@cern.ch)
--
-- date of creation: 20-09-2016
--
-- version: 1.0
--
-- description:
--    Simulation testbench for the new burst mode module to run in as part of the
--    CONV-TTL-BLO common gateware.
--      
--
-- dependencies:
--    None.
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    20-09-2016   Denia Bouhired     File created

--==============================================================================
-- TODO: -
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
use std.textio.all;


--use work.gencores_pkg.all;
--use work.wishbone_pkg.all;

use work.conv_common_gw_pkg.all;

entity testbench is
end entity testbench;

architecture behav of testbench is


  -- Clock periods
  constant c_clk_20_per   : time := 50 ns;
  constant random_intervals : boolean := false;
  
  component conv_dyn_burst_ctrl is
	generic
	(
		g_pwidth : natural range 2 to 40 	:= 5; 
		g_temp_decre_step : t_temp_decre_step := (0, 769, 31, 104, 14, 82, 0 ,0, 0, 0, 0, 0, 0, 0, 0);
		g_1_pulse_temp_rise			:in unsigned (19		downto 0);
		g_max_temp			:in unsigned (39 downto 0) 
	);
	port
	(
    -- Clock and active-low reset inputs
    clk_i        	: in  std_logic;
    rst_n_i      	: in  std_logic;
	
	-- Enable input, pulse generation is enabled when '1'
    en_i         	 	: in  std_logic;
	pulse_burst_i 		: in std_logic;
	pulse_burst_o		: out std_logic;
	
	-- Burst error output, pulses high for one clock cycle when a pulse arrives
    -- within a burst rejection phase
    burst_err_p_o : out std_logic	
	
	);
  end component conv_dyn_burst_ctrl;
  
  
  -- Signal declarations
  
  signal clk_20			: std_logic;
  signal rst			: std_logic;
  
  signal en				: std_logic;
  signal burst_train	: std_logic;
  --signal burst_train_dyn	: std_logic;
  signal burst_train_regulated : std_logic;
  signal burst_train_regulated_dyn : std_logic;
  signal rand_num 		: integer := 0;
  
  
	signal t_start : TIME := NOW;
	signal t_sim1 :TIME := 0 ns;
	signal t_sim2 :TIME := 0 ns;
  --==============================================================================
--  architecture begin
--==============================================================================

  begin
  
-- ===========================================================================
  --Instantiate the DUT: Burst controller averaged over 1000 pulses
  --============================================================================

  
   -- burst_train_dyn <= burst_train;
    --============================================================================
  -- Instantiate the DUT: Dynamic
  --============================================================================
  cmp_dut : conv_dyn_burst_ctrl
  generic map
  (
  g_pwidth 					=> 5, --
  
  --g_temp_decre_step		=> (0,0,0,0,0,0,0,5750,100,79,13,12,4,5,13),
  g_temp_decre_step			=> (0, 769, 31, 104, 14, 82, 0 ,0, 0, 0, 0, 0, 0, 0, 0),
  g_1_pulse_temp_rise		=> x"01388",
  --g_1_pulse_temp_rise		=> x"23040", -- for 1.2us pulse
  g_max_temp			=> x"00000F4240" --10^6
 
  --g_max_temp			=> x"02540BE400" --10^10
  )
  port map(
  clk_i			=> clk_20,
  rst_n_i		=>  rst,
  
  en_i				=> en,
  pulse_burst_i		=> burst_train,
  
  pulse_burst_o		=> burst_train_regulated_dyn,
  burst_err_p_o		=> open
  
  );
  
  
   --============================================================================
-- Generate clock signals
--============================================================================
  p_clk_20 : process
  begin
    clk_20 <= '1';
    wait for c_clk_20_per/2;
    clk_20 <= '0';
    wait for c_clk_20_per/2;
  end process p_clk_20;
  --============================================================================ 
  -- Random number generator
   --============================================================================
 p_ran_gen : process 
    variable seed1, seed2: positive := 1;               -- seed values for random generator
    variable rand: real;   -- random real-number value in range 0 to 1.0  
    variable range_of_rand : real := 1000.0;    -- the range of random values created will be 0 to +10000.
begin

    uniform(seed1, seed2, rand);   -- generate random number
    rand_num <= integer(rand*range_of_rand);  -- rescale to 0..1000, convert integer part 
    wait for 1000 ns;
end process p_ran_gen;


  
  process 
  begin
 
	rst <= '1';
	wait for 2500 ns;
	rst <= '0';
	wait for 2000 ns;
	rst <= '1';
	wait;
  end process;  
  
  process 
  begin
	en <= '0';
	wait for 5000 ns;
	en <= '1';
	wait;
  end process; 
  
  
  --============================================================================
  -- Pulse stimuli
  --============================================================================
  
  p_stim_burst1 : process
    variable interval	: time;-- := 1000 ns;
	
  begin
  while t_sim1 < 50000 us loop
 --while true loop
	t_sim1 <=  NOW - t_start;
	if random_intervals then
		interval := rand_num * 1 ns;
		if interval < 250 ns then
			interval := 250 ns;
		end if;
	else	
		interval := 250 ns;--changes pulse frequency
	end if;
	burst_train <= '0';
	wait for interval;
	burst_train <= '1';
	wait for 250 ns;-- changes pulse width
	burst_train <= '0';
  end loop;  
   
    while t_sim2 < 50000 us loop
	t_sim2 <=  NOW - t_sim1;
	if random_intervals then
		interval := rand_num * 1 ns;
	else	
		interval := 1750 ns;
	end if;
	burst_train <= '0';
	wait for interval;
	burst_train <= '1';
	wait for 250 ns;
	burst_train <= '0';
  end loop; 
   
  end process p_stim_burst1;
  
  
  -- p_write_output : process (temp_rise_counter)
  -- file F 	 : text open write_mode is "C:\Users\debouhir\work\CONV-TTL-BLO\conv-ttl-blo\conv-ttl-blo-gw\sim\Release\temp_rise_counter.txt";
  -- variable L : line;
  -- begin
	-- write (L, NOW, left, 30);
	-- write (L, to_integer((temp_rise_counter (19 downto 0))), left, 50);
	-- write (L, to_integer((temp_rise_counter (39 downto 20))), left, 50);
	-- writeline (F, L);
  -- end process p_write_output;
  
  end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================

  
  
  
  
  